
#include 'Error.ch'
#include 'Xbp.ch'
#include 'XbpDev.ch'
#define  CR                      Chr(13)
#define  LF                      Chr(10)
#define  CRLF                    Chr(13) + Chr(10)
 ////////////
 //   PrtFuncs.prg
 //            Funes comuns ao uso da impressora
 /////

 ////////////
 //
FUNCTION PrinterError_(bError, oPrinter, cUser, cMsgId, cCallStack)
 //      Processa anlise de erros que podem ter ocorrido com a impressora, onde:
 //               <bError>      -> objeto da classe Error()
 //               [<oPrinter>]  -> objeto da classe XbpPrinter() onde ocorreu o erro
 //               <cUser>       -> nome do usurio
 //               <cMsgId>      -> pequena identificao para o ttulo da mensagem. Ser seguida
 //                                de uma mensagem informativa
 //              [<cCallStack>] -> string contendo o call stack para ser exibido em caso da
 //                                impresso no poder continuar
 //
 //      Vide \MSI\GUI\Print\Prgs\Printfor.prg
 //
 //      Retorna: .t. se o erro foi corrigido
 //               .f. se o erro no foi corrigido
 //      Em caso do erro puder ter sido recuperado retorna .t.
 /////
LOCAL  lOk, cMens

lOk := .f.
do while .t.
                  //
                  //    Existe o XbpPrinter() e ocorreu erro com a impressora
   if     oPrinter                    <> NIL
          if oPrinter:printerStatus() <> XBPPRN_STATUS_READY
             lOk                      := PrinterStatus_(oPrinter, cUser, cMsgId)
          else
             lOk                      := .t.
          endif
          exit
   elseif (bError:genCode == XPP_ERR_PRINT               .or.;
           bError:genCode == XPP_ERR_PRINTER_NA          .or.;
          (bError:genCode == XPP_ERR_OPEN       .and.;
           (('OUT' $ Upper(bError:operation))   .or.;
            ('POS' $ Upper(bError:operation))))          .or.;
          (bError:genCode == XPP_ERR_WRITE      .and.;
           (('OUT' $ Upper(bError:operation))   .or.;
            ('POS' $ Upper(bError:operation)))))
                  //
                  //    Erro com a impressora no XbpPrinter()
          cMens := PrinterMSG_(bError, cUser, oPrinter)
          if ConfirmBox(NIL, (cMens   + CR + CR + 'TENTAR NOVAMENTE?'),;
                              (cMsgId +           ' PROBLEMA COM A IMPRESSO'),;
                              XBPMB_RETRYCANCEL,;
                              (XBPMB_QUESTION + XBPMB_SYSMODAL)) == 1
                lOk := .t.
          else
             if !Empty(cCallStack)
                cMens += (CR + cCallStack)
             endif
             ConfirmBox(NIL, cMens,;
                             (cMsgId + ' ERRO: IMPRESSO INTERROMPIDA'),;
                             XBPMB_CANCEL,;
                             (XBPMB_CRITICAL + XBPMB_SYSMODAL))
             exit
          endif
   endif
                  //
                  //    O erro no ocorreu na impressora
   cMens := (PrinterMSG_(bError, cUser, oPrinter:devName) +;
             CR + CR + cCallStack + CR + CR +;
             'Favor enviar o "print screen" desta tela para:' + CR +;
             'suporte@sistemapointer.com')
   ConfirmBox(NIL, cMens,;
                   (cMsgId + ' ERRO: IMPRESSO INTERROMPIDA'),;
                   XBPMB_CANCEL,;
                   (XBPMB_CRITICAL + XBPMB_SYSMODAL))
   exit
enddo
RETURN lOk

 ////////////
 //
STATIC FUNCTION PrinterMSG_(oError, cUser, cPrinter)
 //      Usada para preparar as mensagens de erro que ocorrem ou para a preparao ou antes da
 //            impresso, onde:
 //            <oError>   -> objeto da classe Erro()
 //            <cUser>    -> nome do usurio para identificao nas mensagens
 //            <cPrinter> -> dispositivo (nome) da impressora que apresentou o problema
 //
 //      Retorna string contendo a mensagem de erro.
 //////
LOCAL  cMens, cError
cMens := (cUser + ' IMPRIMINDO EM: "' + cPrinter + '". OCORREU O PROBLEMA:' + CR)

if     oError:genCode == XPP_ERR_PRINT
       cMens          += (CR + 'Erro durante a impresso.')
elseif oError:genCode == XPP_ERR_PRINTER_NA
       cMens          += (CR + 'No est instalada ou no est disponvel')
elseif oError:genCode == XPP_ERR_OPEN
       cMens          += (CR + 'No se conseguiu "abrir" o Arquivo')
       if !Empty(oError:fileName)
          cMens += (': ' + oError:fileName)
       endif
elseif oError:genCode == XPP_ERR_WRITE
       cMens          += (CR + 'No se conseguiu "gravar" no Arquivo')
       if !Empty(oError:fileName)
          cMens += (': ' + oError:fileName)
       endif
else
       cMens          += (CR + 'GenCode: ' + LTrim(Str(oError:genCode)))
endif
if !Empty(oError:description)
   cMens += (CR + 'Descrio: ' + oError:description)
endif
if !Empty(oError:operation)
   cMens += (CR + 'Operao: '  + oError:operation)
endif
if !Empty(oError:args)
   cMens += (CR + 'Parmetro: "' + Arg2Char_(oError:args) + '"')
endif
if !Empty(oError:osCode)
   cError := DosErrorMessage(oError:osCode)
   cMens  += (CR + 'Erro indicado pelo Windows: ' + cError)
endif
RETURN cMens

 ////////////
 //
FUNCTION PrinterStatus_(oPrint, cUserName, cMsgId)
 //      Verifica a disponibilidade da impressora, onde:
 //                <oPrint>      -> impressora criada com XbpPrinter():new():create(...)
 //               [<cUserName>]  -> nome para identificar o usurio na mensagem
 //               [<cMsgId>]     -> nome para identificar a origem da mensagem no ttulo.
 //                                 Ex.: 'POINTER'
 //      O aplicativo tem de testar oPrint:status() == XBPSTAT_CREATE antes  executar esta funo.
 //
 //      Caso seja detetado algum problema  oferecida ao operador uma opo de
 //           tentar novamente.
 //
 //
 //      Retorna: .t. se tudo ok
 //               .f. se foi detetado algum problema que o operador no corrigiu
 /////
LOCAL  lOk, nStatus, cUser, cTitle, cMens, oApp, oHadFocus

do while .t.
   lOk        := .t.
   nStatus    := oPrint:printerStatus()
   if nStatus == XBPPRN_STATUS_READY
      exit
   endif
   if cUserName == NIL
      cUser     := 'A '
   else
      cUser := (cUserName + ': a ')
   endif
   if     !Empty(oPrint:devname)
          cUser += ('IMPRESSORA: ' + oPrint:devName + ' ')
   elseif !Empty(oPrint:location)
          cUser += ('IMPRESSORA localizada em: ' + oPrint:location + ' ')
   elseif !Empty(oPrint:devPort)
          cUser += ('IMPRESSORA da porta: ' + oPrint:devPort + ' ')
   else
          cUser += ('IMPRESSORA: ??? ')
   endif
   if cMsgId == NIL
      cTitle := ''
   else
      cTitle := (cMsgId + '.')
   endif
   cTitle         += ' ERRO DURANTE IMPRESSO'
   if     nStatus == XBPPRN_STATUS_PAUSED
          cMens   := 'EST PAUSADA.'
   elseif nStatus == XBPPRN_STATUS_BUSY
          cMens   := 'EST OCUPADA.'
   elseif nStatus == XBPPRN_STATUS_SERVICE
          cMens   := ('APRESENTA UM PROBLEMA RELATIVO A:'             + CR + CR +;
                      'a) est com pouco ou sem toner'                     + CR +;
                      'b) a bandeija de papel est cheia'                  + CR +;
                      'c) com o tracionador de papel'                      + CR +;
                      'd) no identificado.')
   elseif nStatus == XBPPRN_STATUS_NOPAPER
          cMens   := 'EST SEM PAPEL.'
   elseif nStatus == XBPPRN_STATUS_NOT_READY
          cMens   := ('APRESENTA UM PROBLEMA RELATIVO A:'             + CR + CR +;
                      'a) a porta est aberta'                             + CR +;
                      'b) ainda no est preparada.')
   elseif nStatus == XBPPRN_STATUS_NA
          cMens   := 'NO EST DISPONVEL.'
   elseif nStatus == XBPPRN_STATUS_ERROR
          cMens   := ('APRESENTA UM PROBLEMA RELATIVO A:'             + CR + CR +;
                      'a) memria insuficiente'                            + CR +;
                      'b) no consegue imprimir.')
   elseif nStatus == XBPPRN_STATUS_OFFLINE
          cMens   := 'EST SEM ACESSO.'
   else
          cMens   := ('o DRIVER informa cdigo desconhecido de erro:' + CR + CR +;
                      AllTrim(Transform(nStatus, '@'))                     + CR +;
                      'consulte o Manual da Impressora.')
   endif
   cMens     := (cUser + CR + CR + cMens)
   oApp      := SetAppWindow()
   oHadFocus := SetAppFocus()
   if ConfirmBox(NIL, (cMens + CR + CR + 'TENTAR NOVAMENTE?'),;
                       cTitle,;
                       XBPMB_RETRYCANCEL,;
                       (XBPMB_QUESTION + XBPMB_SYSMODAL),;
                       XBPMB_DEFBUTTON1) == XBPMB_RET_OK
   else
      lOk := .f.
   endif
   SetAppWindow(oApp)
   SetAppFocus(oHadFocus)
   exit
enddo
RETURN lOk